@@ -84,6 +84,8 @@
 # Host *
 #   IdentitiesOnly yes
 #   ServerAliveInterval 100
+import string
+import random
 import json
 import hashlib
 import time
@@ -205,6 +207,8 @@ def get_node(repopath, node, snapshot, ref, connector):
 def get_all_nodes(repopath, node, snapshot, ref, connector):
     """ Fetch all the nodes from the cloned git repository in the given path."""
     db_records = []
+    charVal = (random.choice(string.ascii_letters) for x in range(4))
+    randomstr = ''.join(charVal)
     collection = node['collection'] if 'collection' in node else COLLECTION
     given_type = get_field_value(connector, "type")
     base_path = get_field_value_with_default(connector, "folderPath", "")
@@ -237,7 +241,7 @@ def get_all_nodes(repopath, node, snapshot, ref, connector):
             logger.info('type: %s, json:%s', node_type, json_data)
             if json_data:
                 db_record = copy.deepcopy(d_record)
-                db_record['snapshotId'] = '%s%s' % (node['masterSnapshotId'], str(count))
+                db_record['snapshotId'] = '%s%s%s' % (node['masterSnapshotId'], randomstr, str(count))
                 db_record['path'] = path.replace('//', '/')
                 db_record['contentType'] = contentType
                 db_record['json'] = json_data
